from psydk import run_experiment
from psydk.visual.geometry import Transformation2D, Shape
from psydk.visual.stimuli import ShapeStimulus, GaborStimulus, ImageStimulus, PatternStimulus
import pymunk
import sys

class Ball:
    def __init__(self, x, y, radius, mass, space, randomize=True):
        inertia = pymunk.moment_for_circle(mass, 0, radius, (0, 0))
        self.body = pymunk.Body(mass, inertia)
        self.body.position = x, y
        # if randomize, add a random velocity
        if randomize:
            self.body.apply_impulse_at_local_point((0.1, 0))
        self.shape = pymunk.Circle(self.body, radius, (0, 0))
        self.shape.elasticity = 0.3
        self.shape.friction = 0.8
        space.add(self.body, self.shape)
        self.stim = ShapeStimulus(Shape.circle(radius), x=x, y=y, fill_color=(1, 0, 0, 1))
        self.space = space

    def update(self):
        self.stim["x"] = self.body.position.x
        self.stim["y"] = self.body.position.y

    def remove(self):
        self.space.remove(self.body, self.shape)

def my_experiment(exp_manager) -> None:
    # create a new window
    #
    print(__renderer_factory)

    main_window = exp_manager.create_default_window(fullscreen=True, monitor=1)

    space = pymunk.Space()      # Create a Space which contain the simulation
    space.gravity = 0,98.1 * 2      # Set its gravity
    space.damping = 0.999

    balls = []

    main_window.add_event_handler("mouse_button_press", lambda e: balls.append(Ball(e.position[0], e.position[1], 25, 1, space)))

    # add the ground
    ground_x, ground_y, ground_width = -250, 500, 500
    shape = pymunk.Segment(space.static_body, (ground_x, ground_y), (ground_x + ground_width, ground_y), 0.0)
    shape.elasticity = 0.9999999
    shape.friction = 0.9
    space.add(shape)

    ground_stim = ShapeStimulus(Shape.line(ground_x, ground_y, ground_x + ground_width, ground_y), stroke_color=(0, 0, 0, 1))

    # the background
    # background = PatternStimulus(Shape.rectangle(width="1sw", height="1sh"), x="-0.25sw", y=-550, pattern="checkerboard", stroke_color="red", fill_color=(0.0, 0.0, 0.0, 1.0))

    background = PatternStimulus(Shape.circle(600), pattern="checkerboard", stroke_color="red", fill_color=(0.0, 0.0, 0.0, 1.0))


    for i in range(10000000):
        frame = main_window.get_frame()

        frame.draw(background)

        for i, ball in enumerate(list(balls)):
            # check if the ball is out of the screen
            if ball.body.position.y < -500:
                space.remove(ball.body, ball.shape)
                balls.pop(i)
                continue
            ball.update()
            frame.draw(ball.stim)

        space.step(0.02)


        frame.draw(ground_stim)


        main_window.present(frame)

if __name__ == "__main__":
    run_experiment(my_experiment)



from psydk import run_experiment
from psydk.visual.geometry import Transformation2D, Shape
from psydk.visual.stimuli import ShapeStimulus, GaborStimulus, ImageStimulus
import sys

def my_experiment(exp_manager) -> None:
    # create a new window

    main_window = exp_manager.create_default_window(fullscreen=True, monitor=2)

    input_circles = {}


    def circle_move(event):
        print(event)
        x, y = event.position
        id = event.id if event.id is not None else 0

        if id not in input_circles:
            input_circles[id] = ShapeStimulus(Shape.circle(0, 0, "0.01sw"), x=-100, fill_color=(1, 0, 0, 1))

        input_circles[id]["x"] = x
        input_circles[id]["y"] = y

    # def circle_click(event):
    #     circle["fill_color"] = "yellow"

    # def circle_release(event):
    #     circle["fill_color"] = "blue"

    main_window.add_event_handler("key_press", lambda e: sys.exit(0) if e.key == "Q" else None)
    main_window.add_event_handler("cursor_moved", circle_move)
    main_window.add_event_handler("touch_move", circle_move)
    # main_window.add_event_hanxdler("mouse_button_press", circle_click)
    # main_window.add_event_handler("mouse_button_release", circle_release)

    event_receiver = main_window.create_event_receiver()

    # rect0 = ShapeStimulus(Shape.rectangle("-0.5sw", "-0.5sh", "1sw", "0.5sh"), fill_color=(1, 0, 0, 1))
    rect1 = ShapeStimulus(Shape.rectangle("-0.5sw", "-0.5sh", "1sw", "1sh"), fill_color=(0, 0, 0, 1))
    image = ImageStimulus("test.png", "-0.25sw", "-0.25sh", "0.25sw", "0.25sw", anchor = "center")
    rect2 = ShapeStimulus(Shape.rectangle(0, 0, "0.25sw", "0.25sw"), stroke_color=(1, 0, 0, 1), stroke_width=10)
    gabor = GaborStimulus(0, 0, "0.25sw", 70, 50, anchor = "center", stroke_style="Solid", stroke_width=5)

    is_visible = False
    is_visible2 = False

    for i in range(10000000):
        frame = main_window.get_frame()


        angle = (i / 10) % 360

        gabor.rotated_at(angle, 0, 0)

        rect2.rotated_at(-angle, 0, 0)
        # image.rotated_at(-angle, 0, 0)


        keys = event_receiver.poll()

        if "Enter" in keys.keys_pressed():
            is_visible = True

        if "Enter" in keys.keys_released():
            is_visible = False

        if is_visible:
            frame.draw(rect1)

        # if is_visible2:
        #     frame.draw(rect0)

        frame.draw(gabor)
        frame.draw(image)
        frame.draw(rect2)

        for circle in input_circles.values():
            frame.draw(circle)

        is_visible2 = not is_visible2

        main_window.present(frame)

if __name__ == "__main__":
    run_experiment(my_experiment)



# import the contents of the Rust library into the Python extension
print("importing psydk")
from .psydk import *
from .psydk import __all__

# optional: include the documentation from the Rust module
from .psydk import __doc__  # noqa: F401

# set gstreamer plugin environment variable to site-packages/psydk/.dylibs/
import platform

if platform.system() == 'Darwin':
    import os
    import sys
    path = os.path.join(os.path.dirname(os.path.abspath(__file__)), ".dylibs")
    os.environ["GST_PLUGIN_PATH"] = path + ":" + os.environ.get("GST_PLUGIN_PATH", "")
